//
//  UKTGrabber.m
//  UK Transport
//
//  Created by Amy Worrall on 13/10/2013.
//  Copyright (c) 2013 Double and Thrice. All rights reserved.
//

#import "UKTGrabber.h"

@interface UKTGrabber ()<NSURLConnectionDataDelegate>

@property (nonatomic, assign) BOOL isExecuting;
@property (nonatomic, assign) BOOL isFinished;
@property (nonatomic, strong) NSURLConnection *connection;
@property (nonatomic, strong) NSMutableData *buffer;
@property (nonatomic, strong) NSData *data;

@end

@implementation UKTGrabber



- (void)start
{
  NSURL *url = [self urlToFetch];

  NSMutableURLRequest* request = [NSMutableURLRequest requestWithURL:url cachePolicy:NSURLRequestReloadIgnoringLocalCacheData timeoutInterval:60];
  self.isExecuting = YES;
  self.isFinished = NO;
  self.buffer = [NSMutableData new];

  NSData *body = [self httpBodyToSend];
  if (body)
  {
    request.HTTPBody = body;
    request.HTTPMethod = @"POST";
    [request setValue:[NSString stringWithFormat:@"%lu", (unsigned long)body.length] forHTTPHeaderField:@"Content-Length"];
  }

  NSString *contentType = [self contentTypeHeader];
  if (contentType)
  {
    [request setValue:contentType forHTTPHeaderField:@"Content-Type"];
  }
  [self configureRequest:request];

  self.connection = [[NSURLConnection alloc] initWithRequest:request
                                                    delegate:self startImmediately:NO];


  [self.connection scheduleInRunLoop:[NSRunLoop mainRunLoop]
                             forMode:NSDefaultRunLoopMode];
  [self.connection start];
}

- (void)configureRequest:(NSMutableURLRequest*)req;
{

}

- (BOOL)isConcurrent
{
  return YES;
}

- (void)cancel
{
  [super cancel];
  [self.connection cancel];
  self.isFinished = YES;
  self.isExecuting = NO;
}

- (void)connection:(NSURLConnection *)connection didReceiveData:(NSData *)data
{
  [self.buffer appendData:data];
}

- (void)connection:(NSURLConnection *)connection didFailWithError:(NSError *)error
{
  self.error = error;

  self.isExecuting = NO;
  self.isFinished = YES;
}

- (void)connectionDidFinishLoading:(NSURLConnection *)connection
{
  self.data = self.buffer;
  self.buffer = nil;

  [self parseLoadedData:self.data];

}

- (void)connection:(NSURLConnection *)connection didReceiveAuthenticationChallenge:(NSURLAuthenticationChallenge *)challenge {
  if ([challenge previousFailureCount] == 0) {
    NSURLCredential *newCredential = [NSURLCredential credentialWithUser:self.authUsername
                                                                password:self.authPassword
                                                             persistence:NSURLCredentialPersistenceForSession];
    [[challenge sender] useCredential:newCredential forAuthenticationChallenge:challenge];
  }
  else {
    NSLog(@"previous authentication failure");
  }
}

- (NSString*)authUsername;
{
  return nil;
}

- (NSString*)authPassword;
{
  return nil;
}

- (void)finish;
{
  self.isExecuting = NO;
  self.isFinished = YES;
}

- (void)setIsExecuting:(BOOL)isExecuting
{
  [self willChangeValueForKey:@"isExecuting"];
  _isExecuting = isExecuting;
  [self didChangeValueForKey:@"isExecuting"];
}

- (void)setIsFinished:(BOOL)isFinished
{
  [self willChangeValueForKey:@"isFinished"];
  _isFinished = isFinished;
  [self didChangeValueForKey:@"isFinished"];
}


#pragma mark - Subclasses override

- (NSURL*)urlToFetch;
{
  return [[NSURL alloc] init];
}

- (NSData*)httpBodyToSend;
{
  return nil;
}

- (void)parseLoadedData:(NSData*)data;
{

}

- (NSString*)contentTypeHeader;
{
  return nil;
}


@end

